/*********************************************************************
	Copyright (C) 2002 Mustafa Demirhan :)
	
	This software is provided 'as-is', without any express or implied
	warranty.  In no event will the authors be held liable for any damages
	arising from the use of this software.

	Permission is granted to anyone to use this software for any purpose,
	including commercial applications, and to alter it and redistribute it
	freely, subject to the following restrictions:

	1. The origin of this software must not be misrepresented; you must not
	   claim that you wrote the original software. If you use this software
	   in a product, an acknowledgment in the product documentation would be
	   appreciated but is not required.

	2. Altered source versions must be plainly marked as such, and must not be
	   misrepresented as being the original software.

	3. This notice may not be removed or altered from any source distribution.

	http://www.macroangel.com
	admin@macroangel.com
**********************************************************************/

#include "stdafx.h"
#include <afxdllx.h>
#include "FTPClient.h"
#include "FTPDownload.h"
#include "FTPUpload.h"
#include "ProgressDlg.h"
#include "StringHelper.h"
#include "FileInfo.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


static AFX_EXTENSION_MODULE FTPClientDLL = { NULL, NULL };

extern "C" int APIENTRY
DllMain(HINSTANCE hInstance, DWORD dwReason, LPVOID lpReserved)
{
	// Remove this if you use lpReserved
	UNREFERENCED_PARAMETER(lpReserved);

	if (dwReason == DLL_PROCESS_ATTACH)
	{
		TRACE0("FTPCLIENT.DLL Initializing!\n");
		
		// Extension DLL one-time initialization
		if (!AfxInitExtensionModule(FTPClientDLL, hInstance))
			return 0;

		// Insert this DLL into the resource chain
		// NOTE: If this Extension DLL is being implicitly linked to by
		//  an MFC Regular DLL (such as an ActiveX Control)
		//  instead of an MFC application, then you will want to
		//  remove this line from DllMain and put it in a separate
		//  function exported from this Extension DLL.  The Regular DLL
		//  that uses this Extension DLL should then explicitly call that
		//  function to initialize this Extension DLL.  Otherwise,
		//  the CDynLinkLibrary object will not be attached to the
		//  Regular DLL's resource chain, and serious problems will
		//  result.

		new CDynLinkLibrary(FTPClientDLL);
	}
	else if (dwReason == DLL_PROCESS_DETACH)
	{
		TRACE0("FTPCLIENT.DLL Terminating!\n");
		// Terminate the library before destructors are called
		AfxTermExtensionModule(FTPClientDLL);
	}
	return 1;   // ok
}

void DoEvents()
{
	MSG msg;
	
	// window message         
	while (PeekMessage(&msg,NULL,NULL,NULL,PM_REMOVE))         
	{            
		TranslateMessage(&msg);            
		DispatchMessage(&msg);         
	}      
}

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CFTPClient::CFTPClient ()
{
	// Set the timeout value to 30 seconds
	m_InternetSession.SetOption (INTERNET_OPTION_CONNECT_TIMEOUT, 30);
	m_InternetSession.SetOption (INTERNET_OPTION_RECEIVE_TIMEOUT, 30);
	m_InternetSession.SetOption (INTERNET_OPTION_SEND_TIMEOUT, 30);

	m_pFtpConnection = NULL;
}

CFTPClient::~CFTPClient()
{
	if (m_pFtpConnection != NULL)
	{
		m_pFtpConnection->Close ();
		delete m_pFtpConnection;
		m_pFtpConnection = NULL;
		m_sFTPRootFolder.Empty ();
	}
}

bool CFTPClient::IsInitialized	()
{
	if (m_pFtpConnection != NULL)
		return true;
	else
		return false;
}

/*void CFTPClient::SetProxy (bool bEnabled, const CString &sProxyServer, 
						   const CString &sProxyUser, const CString &sProxyPassword)
{
	m_InternetSession.SetOption (INTERNET_OPTION_PROXY, );		
}*/


void CFTPClient::Login (const CString &sHost, const CString &sUsername, 
						const CString &sPassword, BOOL bPASV, int nPort, 
						int nRetries, int nRetryDelay)
{
	if (m_pFtpConnection != NULL)
		throw new CFTPException (CFTPException::FTP_AlreadyConnected, "Another session is not disconnected yet. Logoff before trying to connect again!");
	
	
	HANDLE hEventKill = CreateEvent (NULL, TRUE, FALSE, NULL);
	
	// set animation to 'Downloading'
	CProgressDlg dlg;
	dlg.m_nAnimationID = IDR_AVI1;
	
	// Create the progress dialog box.
	if (!dlg.Create (NULL, hEventKill))
	{
		ASSERT (FALSE);
		throw new CFTPException (CFTPException::FTP_Unknown, "Unable to create the progress dialog. Please contact the program vendor for more information.");
	}
	
	CString sStatus;
	int nRetryCount = 0;

	while (nRetryCount < nRetries)
	{
		nRetryCount ++;
		try
		{
			// Create new ftp connection to retrieve file
			sStatus.Format ("Connecting to %s (Attempt %d of %d)", sHost, nRetryCount, nRetries);
			dlg.SetStatus (sStatus);
			
			// Connect to FTP Server
			m_pFtpConnection = m_InternetSession.GetFtpConnection (sHost, sUsername, sPassword, nPort, bPASV);
			
			m_pFtpConnection->GetCurrentDirectory (m_sFTPRootFolder);
			StringHelper::RemoveWSFromFileName (m_sFTPRootFolder);
			StringHelper::AddBSToFolderName (m_sFTPRootFolder, '/');
			m_sFTPRootFolder.Replace	("\\", "/");

			// Connected! Exit the while loop.
			break;
		}
		catch (CInternetException* pEx)
		{
			m_pFtpConnection = NULL;
			
			// catch errors from WinINet
			pEx->GetErrorMessage (sStatus.GetBuffer (MAX_PATH), MAX_PATH);
			pEx->Delete ();
			sStatus.ReleaseBuffer ();
			
			if (nRetryCount < nRetries)
			{
				sStatus.Format ("Failed to connect to %s (Attempt %d of %d)", sHost, nRetryCount, nRetries); 
				dlg.SetStatus(sStatus);
				
				sStatus.Format ("Retrying after %d seconds...", nRetryDelay); 
				dlg.SetSecondStatus(sStatus);
				
				DWORD dwRet;
				DWORD dwMaxTick = GetTickCount () + nRetryDelay * 1000;
				
				while (1)
				{
					// wait for event or message, if it's a message, process it and return to waiting state
					dwRet = MsgWaitForMultipleObjects (1, &hEventKill, FALSE, dwMaxTick - GetTickCount (), QS_ALLINPUT);
					if (dwRet == WAIT_OBJECT_0)
					{
						// user canceled
						throw new CFTPException (CFTPException::FTP_UserAborted, "Login is aborted by the user");
					}   
					else if (dwRet == WAIT_OBJECT_0 + 1)
					{
						DoEvents ();
					}  
					else
					{
						// timed out!
						break;
					}
				}
				dlg.SetSecondStatus ("");
			}
			else
			{
				throw new CFTPException (CFTPException::FTP_LoginFailed, "Cannot login to the server " + sHost);
			}
		}
	}
	
	dlg.DestroyWindow ();
}

void CFTPClient::Logoff ()
{
	if (m_pFtpConnection == NULL)
		throw new CFTPException (CFTPException::FTP_NotConnected, "Not connected yet!");
	else
	{
		m_pFtpConnection->Close ();
		delete m_pFtpConnection;
		m_pFtpConnection = NULL;
		m_sFTPRootFolder.Empty ();
	}
}

void CFTPClient::Download (const CString &sRemoteFiles, const CString &sLocalFolder, bool *pbAbort, bool bRecursive, DWORD dwTransferType)
{
	if (m_pFtpConnection == NULL)
		throw new CFTPException (CFTPException::FTP_NotConnected, "Not connected to any FTP server!");
	
	CFTPDownload ftpDownload (this);
	ftpDownload.m_sRemoteName = sRemoteFiles;
	ftpDownload.m_sLocalFolder = sLocalFolder;
	RemoveFirstBS (ftpDownload.m_sRemoteName);
	RemoveFirstBS (ftpDownload.m_sLocalFolder);
	ftpDownload.m_sFTPRootFolder = m_sFTPRootFolder;
	ftpDownload.DownloadAllFiles (pbAbort, bRecursive);
}

void CFTPClient::Upload (const CString &sLocalFiles, const CString &sRemoteFolder, bool *pbAbort, bool bRecursive, DWORD dwTransferType)
{
	if (m_pFtpConnection == NULL)
		throw new CFTPException (CFTPException::FTP_NotConnected, "Not connected to any FTP server!");
	
	CFTPUpload ftpUpload (this);
	ftpUpload.m_sLocalName = sLocalFiles;
	ftpUpload.m_sRemoteFolder = sRemoteFolder;
	RemoveFirstBS (ftpUpload.m_sLocalName);
	RemoveFirstBS (ftpUpload.m_sRemoteFolder);
	ftpUpload.m_sFTPRootFolder = m_sFTPRootFolder;
	ftpUpload.UploadAllFiles (pbAbort, bRecursive);
}

// sRemoteFolder is the folder name relative to the
// root folder!!!
void CFTPClient::CreateMultiFolder (const CString &sOrgRemoteFolder)
{
	if (m_pFtpConnection == NULL)
		throw new CFTPException (CFTPException::FTP_NotConnected, "Not connected to any FTP server!");

	CString sTemp;
	CStringArray sFolderList; // CString Array to hold Directory Structures

	CString sRemoteFolder (sOrgRemoteFolder);
	RemoveFirstBS (sRemoteFolder);
	
	// Separate the String into its compounded components
	for (int i = 0; i < sRemoteFolder.GetLength (); i++ )
	{									
		if (sRemoteFolder.GetAt (i) != '\\' && sRemoteFolder.GetAt (i) != '/')
		{
			sTemp += sRemoteFolder.GetAt (i);
			
			if (i == sRemoteFolder.GetLength () - 1)
				sFolderList.Add (sTemp);
		}
		else
		{
			sFolderList.Add (sTemp);
			sTemp += "/";
		}
	}
	
	// Now lets cycle through the String Array and create each directory in turn
	for (i = 0; i < sFolderList.GetSize (); i++)
	{
		sTemp = m_sFTPRootFolder + sFolderList.GetAt (i);

		CFtpFileFind ftpFind (m_pFtpConnection);
		
		if (!ftpFind.FindFile (sTemp))
		{
			// The directory is not found. Lets create it!
			if (!m_pFtpConnection->CreateDirectory (sTemp))
				throw new CFTPException (CFTPException::FTP_CreateFolderFailed, "Cannot create folder " + m_sFTPRootFolder + sTemp);
		}
		ftpFind.Close ();
	}
}

void CFTPClient::DeleteFiles (const CString &sOrgRemoteFiles, bool *pbAbort, bool bRecursive, bool bAbortIfFailsOnce)
{
	if (m_pFtpConnection == NULL)
		throw new CFTPException (CFTPException::FTP_NotConnected, "Not connected to any FTP server!");

	CString sRemoteFiles (sOrgRemoteFiles);
	RemoveFirstBS (sRemoteFiles);
	DeleteRemoteFiles (false, sRemoteFiles, pbAbort, bRecursive, bAbortIfFailsOnce);
}

void CFTPClient::DeleteFolders (const CString &sOrgRemoteFolder, bool *pbAbort, bool bRecursive, bool bAbortIfFailsOnce)
{
	if (m_pFtpConnection == NULL)
		throw new CFTPException (CFTPException::FTP_NotConnected, "Not connected to any FTP server!");

	CString sRemoteFolder (sOrgRemoteFolder);
	RemoveFirstBS (sRemoteFolder);
	DeleteRemoteFiles (true, sRemoteFolder, pbAbort, bRecursive, bAbortIfFailsOnce);
}

// If bDeleteDirs = true, then only the directories are deleted
// If bDeleteDirs = false, then only the files are deleted
// To delete both of them, you should first delete all of the files
// and then delete all of the folders (by this way, the folders are empty)
void CFTPClient::DeleteRemoteFiles (bool bDeleteDirs, const CString &sOrgRemoteFiles, bool *pbAbort, bool bRecursive, bool bAbortIfFailsOnce)
{
	if (m_pFtpConnection == NULL)
		throw new CFTPException (CFTPException::FTP_NotConnected, "Not connected to any FTP server!");

	try	
	{
		BOOL bSuccess = TRUE;
		CString sRemoteFiles (sOrgRemoteFiles);
		StringHelper::RemoveWSFromFileName (sRemoteFiles);
		sRemoteFiles.Replace		("\\", "/");

		CString sRemoteWildcard, sBaseRemoteFolder;
		SeperateFilename (sRemoteFiles, sBaseRemoteFolder, sRemoteWildcard, '/');
		if (sRemoteWildcard.IsEmpty ())
		{
			// WOW. Somebody want to delete the root folder.
			// Should I allow????? Hmmm. For now, I will NOT!!!
			if (bDeleteDirs && sBaseRemoteFolder.IsEmpty ())
				throw new CFTPException (CFTPException::FTP_DeleteRootFolderNotAllowed, "Deletion of the root folder is not allowed!");
		}
		
		// if a single file or folder!
		if (sRemoteWildcard.FindOneOf ("*?") == -1)
		{
			if (!bDeleteDirs)
				bSuccess = m_pFtpConnection->Remove (m_sFTPRootFolder + sRemoteFiles);
			else
				bSuccess = m_pFtpConnection->RemoveDirectory (m_sFTPRootFolder + sRemoteFiles);
		}
		// For multiple files!
		else
		{
			HANDLE	hEventKill = CreateEvent (NULL, TRUE, FALSE, NULL);
			
			CProgressDlg progressDlg;
			// set animation to 'Downloading'
			progressDlg.m_nAnimationID = IDR_DOWNLOAD_ANIMATION;
			
			// Create the progress dialog box.
			if (!progressDlg.Create (NULL, hEventKill))
			{
				ASSERT (FALSE);
				throw new CFTPException (CFTPException::FTP_Unknown, "Unable to open create the progress dialog. Please contact the program vendor for more information.");
			}

			progressDlg.SetWindowTitle ("Receiving remote directory listing...");
			progressDlg.SetSecondStatus ("");
			progressDlg.SetStatus ("Receiving remote directory listing...");
			progressDlg.SetPos (0);

			CStringArray sRemoteFileList;
			if (!bDeleteDirs)
				GetFileList (m_sFTPRootFolder + sBaseRemoteFolder, true, sRemoteFileList, bRecursive);
			// DON'T get recursive listing for folders. It will automatically recurse if needed!
			else
				GetFileList (m_sFTPRootFolder + sBaseRemoteFolder, true, sRemoteFileList, false);

			progressDlg.SetUpper (sRemoteFileList.GetSize ());
			
			CString sRemoteFileName, sRemoteFolder, sRemotePath;

			progressDlg.SetWindowTitle ("Deleting files...");
			progressDlg.SetStatus ("Deleting files...");
			
			for (int i = 0; i < sRemoteFileList.GetSize (); i++)
			{
				if (*pbAbort)
					break;
				
				if (WaitForSingleObject (hEventKill, 0) != WAIT_TIMEOUT)
					throw new CFTPException (CFTPException::FTP_UserAborted, "File/Folder deletion is aborted by the user");
				
				AfxExtractSubString (sRemoteFileName,	sRemoteFileList.GetAt(i), 0, '|');
				AfxExtractSubString (sRemoteFolder,		sRemoteFileList.GetAt(i), 1, '|');
				
				// If the filename is not empty, then this is not 
				// a directory. It is a file. Now, delete the file
				// if we are supposed to delete the files
				if ((!sRemoteFileName.IsEmpty ()) && (!bDeleteDirs))
				{
					sRemotePath = sRemoteFolder;
					StringHelper::AddBSToFolderName (sRemotePath, '/');
					sRemotePath += sRemoteFileName;
					
					if (StringHelper::WildcardCompare (sRemoteWildcard, sRemotePath))
					{
						progressDlg.SetStatus ("Deleting file " + sRemotePath);
						if (!m_pFtpConnection->Remove (sRemotePath))
						{
							bSuccess = FALSE;
							if (bAbortIfFailsOnce)
								throw new CFTPException (CFTPException::FTP_DeleteFilesFailed, "Failed to delete all of the files!");
						}
					}
				}
				// If this is a directory and we are deleting directories
				else if (sRemoteFileName.IsEmpty () && bDeleteDirs)
				{
					// Dont delete the first item. Its is the root folder
					if ((i != 0) && StringHelper::WildcardCompare (sRemoteWildcard, sRemoteFolder))
					{
						// First delete the folders inside that folder. 
						// Otherwise we won't be able to delete this folder!
						if (bRecursive)
						{
							if (sRemoteFolder.GetLength () > m_sFTPRootFolder.GetLength ())
							{
								CString sTemp;
								if (m_sFTPRootFolder.GetLength () > 0)
								{
									int nLen1 = m_sFTPRootFolder.GetLength ();
									int nLen2 = sRemoteFolder.GetLength ();
									sTemp = sRemoteFolder.Mid (nLen1, nLen2 - nLen1);
									StringHelper::AddBSToFolderName (sTemp, '/');
									sTemp += sRemoteWildcard;
									DeleteRemoteFiles (bDeleteDirs, sTemp, pbAbort, bRecursive, bAbortIfFailsOnce);
								}
								// This cant be. Root folder must be at least "/"
								else
								{
									ASSERT (FALSE);
								}
							}
							else
							{
								ASSERT (FALSE);
							}
						}
						progressDlg.SetStatus ("Deleting folder " + sRemoteFolder);
						if (!m_pFtpConnection->RemoveDirectory (sRemoteFolder))
						{
							bSuccess = FALSE;
							if (bAbortIfFailsOnce)
								throw new CFTPException (CFTPException::FTP_DeleteFoldersFailed, "Failed to delete all of the folders!");
						}
					}
				}

				progressDlg.SetPos (i);
			}
		}
		if (!bSuccess)
			throw new CFTPException (CFTPException::FTP_DeleteFilesFailed, "Failed to delete all of the files!");
	}
	catch (CFTPException * pEx)
	{
		// Dont catch CFTPException's! Let the step catch it...
		throw;
	}
	catch (CException *pEx)
	{
		pEx->Delete ();
		throw new CFTPException (CFTPException::FTP_Unknown, "An unknown error occured");
	}
	catch (...)
	{
		throw new CFTPException (CFTPException::FTP_Unknown, "An unknown error occured");
	}
}

void CFTPClient::GetFileList (const CString &sRemoteFolder, bool bFullPathGiven, CStringArray &sFileNameArray, bool bRecursive, bool bExcludeHidden)
{
	if (m_pFtpConnection == NULL)
		throw new CFTPException (CFTPException::FTP_NotConnected, "Not connected to any FTP server!");

	CFtpFileFind ftpFind (m_pFtpConnection);
	CStringArray sDirectoryArray;
	
	CString sTemp;
	if (bFullPathGiven)
		sTemp = sRemoteFolder;
	else
	{
		sTemp = sRemoteFolder;
		RemoveFirstBS (sTemp);
		sTemp = (m_sFTPRootFolder + sTemp);
	}

	// go to requested directory
	if (!m_pFtpConnection->SetCurrentDirectory (sTemp))
	{
		CString sTemp;
		DWORD dwLength = 255, dwError;
		InternetGetLastResponseInfo (&dwError, sTemp.GetBuffer (dwLength), &dwLength);
		sTemp.ReleaseBuffer ();
		throw new CFTPException (CFTPException::FTP_Remote_SetCurDirFailed, sTemp); 
	}
	
	// add directory (without filename), so we can create the directory structure later
	CString sPath, sFileName;
	
	m_pFtpConnection->GetCurrentDirectory (sPath);
	
	StringHelper::RemoveWSFromFileName (sPath);
	StringHelper::AddBSToFolderName (sPath, '/');
	
	sFileName.Format ("|%s|0", sPath);
	sFileNameArray.Add (sFileName);
	
	BOOL bContinue = ftpFind.FindFile (NULL);//"*");
	if (!bContinue)
	{
		// the directory is empty; just close up and return.
		ftpFind.Close ();
		return;
	}
	
	// add all files in lpszPath
	while (bContinue)
	{
		bContinue = ftpFind.FindNextFile ();
		
		if (ftpFind.IsDots ())
			continue;
		
		// is selected item a directory ?
		if (ftpFind.IsDirectory ())
		{
			sDirectoryArray.Add (ftpFind.GetFilePath ());
			// If not recursive, add the directory names
			if (!bRecursive)
			{
				sFileName.Format ("|%s|0", ftpFind.GetFilePath ());
				sFileNameArray.Add (sFileName);
			}				
		}
		else
		{
			if (! (bExcludeHidden && ftpFind.IsHidden ()))
			{
				sFileName.Format ("%s|%s|%u", ftpFind.GetFileName (), sPath, ftpFind.GetLength ());
				sFileNameArray.Add (sFileName);
			}
		}
	}
	ftpFind.Close ();
	
	if (bRecursive)
	{
		// get contents of directories
		for (int i = 0; i < sDirectoryArray.GetSize(); i++)
			GetFileList (sDirectoryArray.GetAt (i), true, sFileNameArray, bRecursive);
	}
}

// Full name can be a folder name or a full file name (wildcards are allowed)
// The function finds the Folder name and the filename.
// If sFullName ends with a cFolderSeperator, then 
// cFolderSeperator is '/' for FTP and '\\' for local files...
void CFTPClient::SeperateFilename (const CString &sFullName, CString &sFolder, CString &sFilename, TCHAR cFolderSeperator)
{
	if (sFullName.GetLength () > 0)
	{
		if (sFullName.GetAt (sFullName.GetLength () - 1) != cFolderSeperator)
		{
			int nPos = sFullName.ReverseFind (cFolderSeperator);
			if (nPos != -1)
			{
				sFolder = sFullName.Left (nPos + 1);
				sFilename = sFullName.Mid (nPos + 1);
			}
			else
			{
				sFolder = "";
				sFilename = sFullName;
			}
		}
		// If it is a folder name. Thus just add the whole 
		// folder name and set the 
		else
		{
			sFolder = sFullName;
			sFilename = "";
		}
	}
	else
	{
		sFolder = "";
		sFilename = "";
	}
}

void CFTPClient::RemoveFirstBS (CString &sFilename)
{
	int nLength = sFilename.GetLength ();
	if (nLength > 0)
	{
		if (sFilename.GetAt (0) == '\\' ||
			sFilename.GetAt (0) == '/')
		{
			if (nLength > 1)
				sFilename = sFilename.Right (nLength - 1);
			else
				sFilename = "";
		}
	}
}