/*********************************************************************
	Copyright (C) 2002 Mustafa Demirhan :)
	
	This software is provided 'as-is', without any express or implied
	warranty.  In no event will the authors be held liable for any damages
	arising from the use of this software.

	Permission is granted to anyone to use this software for any purpose,
	including commercial applications, and to alter it and redistribute it
	freely, subject to the following restrictions:

	1. The origin of this software must not be misrepresented; you must not
	   claim that you wrote the original software. If you use this software
	   in a product, an acknowledgment in the product documentation would be
	   appreciated but is not required.

	2. Altered source versions must be plainly marked as such, and must not be
	   misrepresented as being the original software.

	3. This notice may not be removed or altered from any source distribution.

	http://www.macroangel.com
	admin@macroangel.com
**********************************************************************/

#if !defined(AFX_FTPCLIENT_H__B24047C4_134F_49AC_80E4_13FBA0828D09__INCLUDED_)
#define AFX_FTPCLIENT_H__B24047C4_134F_49AC_80E4_13FBA0828D09__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

extern void DoEvents ();

class AFX_EXT_CLASS CFTPException : public CException
{
public:
	enum FTPError
	{
		FTP_Unknown,

		FTP_LoginFailed,
		FTP_AlreadyConnected,
		FTP_NotConnected,
		
		FTP_Local_FileCreateFailed,
		FTP_Local_FileOpenFailed,
		FTP_Local_UnknownFileError,

		FTP_Remote_SetCurDirFailed,
		FTP_Remote_FileOpenFailed,
		FTP_Remote_FileWriteFailed,

		FTP_CreateFolderFailed,

		FTP_DeleteFilesFailed,
		FTP_DeleteFoldersFailed,
		FTP_DeleteRootFolderNotAllowed,

		FTP_InvalidLocalFileName,
		FTP_InvalidRemoteFileName,

		FTP_UserAborted
	};
	
	int m_nError;
	CString m_sDesc;
	
	CFTPException (FTPError nError, const CString &sDesc) : CException (TRUE)
	{
		m_nError = nError;
		m_sDesc = sDesc;
	}
};

class AFX_EXT_CLASS CFTPClient  
{
protected:
	CInternetSession m_InternetSession;
	CFtpConnection *m_pFtpConnection;
	CString m_sFTPRootFolder;

	void SeperateFilename (const CString &sFullName, CString &sFolder, CString &sFilename, TCHAR cFolderSeperator);
	void RemoveFirstBS (CString &sFilename);

public:
	CFTPClient ();
	~CFTPClient ();

	bool IsInitialized	();

	void Login			(const CString &sHost, const CString &sUsername, const CString &sPassword, BOOL bPASV = FALSE, int nPort = 21, int nRetries = 1, int nRetryDelay = 10);
	void Logoff			();
	
	void Download		(const CString &sRemoteFiles, const CString &sLocalFolder, bool *pbAbort, bool bRecursive = true, DWORD dwTransferType = FTP_TRANSFER_TYPE_BINARY);
	void Upload			(const CString &sLocalFiles, const CString &sRemoteFolder, bool *pbAbort, bool bRecursive = true, DWORD dwTransferType = FTP_TRANSFER_TYPE_BINARY);
	
	void DeleteFiles	(const CString &sOrgRemoteFiles, bool *pbAbort, bool bRecursive, bool bAbortIfFailsOnce);
	void DeleteFolders	(const CString &sOrgRemoteFolder, bool *pbAbort, bool bRecursive, bool bAbortIfFailsOnce);
	
	void CreateMultiFolder (const CString &sOrgRemoteFolder);

	void GetFileList	(const CString &sRemoteFolder, bool bFullPathGiven, CStringArray &sFileNameArray, bool bRecursive, bool bExcludeHidden = false);

private:
	void DeleteRemoteFiles (bool bDeleteFirs, const CString &sOrgRemote, bool *pbAbort, bool bRecursive, bool bAbortIfFailsOnce);

	friend class CFTPDownload;
	friend class CFTPUpload;
};

#endif // !defined(AFX_FTPCLIENT_H__B24047C4_134F_49AC_80E4_13FBA0828D09__INCLUDED_)
