/*********************************************************************
	Copyright (C) 2002 Mustafa Demirhan :)
	
	This software is provided 'as-is', without any express or implied
	warranty.  In no event will the authors be held liable for any damages
	arising from the use of this software.

	Permission is granted to anyone to use this software for any purpose,
	including commercial applications, and to alter it and redistribute it
	freely, subject to the following restrictions:

	1. The origin of this software must not be misrepresented; you must not
	   claim that you wrote the original software. If you use this software
	   in a product, an acknowledgment in the product documentation would be
	   appreciated but is not required.

	2. Altered source versions must be plainly marked as such, and must not be
	   misrepresented as being the original software.

	3. This notice may not be removed or altered from any source distribution.

	http://www.macroangel.com
	admin@macroangel.com
**********************************************************************/

#include "stdafx.h"
#include "FTPDownload.h"
#include "FTPClient.h"
#include "StringHelper.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


CFTPDownload::CFTPDownload (CFTPClient *pClient)
{
	m_pFTPClient = pClient;

	ASSERT (m_pFTPClient);

	m_dwFileLength = 0;

	// default to binary transfer
	m_dwTransferType = FTP_TRANSFER_TYPE_BINARY;

	m_hEventKill = CreateEvent (NULL, TRUE, FALSE, NULL);
}


CFTPDownload::~CFTPDownload()
{
    m_ProgressDlg.DestroyWindow ();
	CloseHandle (m_hEventKill);
}

void CFTPDownload::DownloadAllFiles (bool *pbAbort, bool bRecursive)
{
	// set animation to 'Downloading'
	m_ProgressDlg.m_nAnimationID = IDR_DOWNLOAD_ANIMATION;
	
	// Create the progress dialog box.
	if (!m_ProgressDlg.Create (NULL, m_hEventKill))
	{
		ASSERT (FALSE);
		throw new CFTPException (CFTPException::FTP_Unknown, "Unable to create the progress dialog. Please contact the program vendor for more information.");
	}
	
	// wait until dialog is created
	WaitForProgressDialog ();

	// catch all exceptions
	try	
	{
		StringHelper::RemoveWSFromFileName (m_sLocalFolder);
		StringHelper::AddBSToFolderName (m_sLocalFolder);
		m_sLocalFolder.Replace		("/", "\\");

		StringHelper::RemoveWSFromFileName (m_sRemoteName);
		m_sRemoteName.Replace		("\\", "/");

		StringHelper::RemoveWSFromFileName (m_sFTPRootFolder);
		StringHelper::AddBSToFolderName (m_sFTPRootFolder, '/');
		m_sFTPRootFolder.Replace	("\\", "/");

		CString sRemoteWildcard, sBaseRemoteFolder;
		m_pFTPClient->SeperateFilename (m_sRemoteName, sBaseRemoteFolder, sRemoteWildcard, '/');
		if (sRemoteWildcard.IsEmpty ())
			sRemoteWildcard = "*";

		// if a single file!
		if (sRemoteWildcard.FindOneOf ("*?") == -1)
		{
			DownloadSingleFile (m_sFTPRootFolder + m_sRemoteName, m_sLocalFolder, pbAbort);
		}
		// For multiple files!
		else
		{
			m_ProgressDlg.SetWindowTitle ("Receiving remote directory listing...");
			m_ProgressDlg.SetSecondStatus ("");
			m_ProgressDlg.SetStatus ("Receiving remote directory listing...");
			m_ProgressDlg.SetPos (0);
			
			CStringArray sRemoteFileList;

			m_pFTPClient->GetFileList (m_sFTPRootFolder + sBaseRemoteFolder, true, sRemoteFileList, bRecursive);
			
			CString sLocalFolder, sRemoteFileName, sRemoteFolder;
			CString sRemotePath, sRemoteFileSize;

			// The first folder is our BASE folder!
			int nReferenceFolder = 0;
			if (sRemoteFileList.GetSize () > 0)
			{
				CString sTemp;
				AfxExtractSubString (sTemp, sRemoteFileList.GetAt (0), 1, '|');
				nReferenceFolder = sTemp.GetLength ();
			}
			
			// Download all files in the queue
			for (int i = 0; i < sRemoteFileList.GetSize (); i++)
			{
				if (*pbAbort)
					break;

				AfxExtractSubString (sRemoteFileName,	sRemoteFileList.GetAt(i), 0, '|');
				AfxExtractSubString (sRemoteFolder,		sRemoteFileList.GetAt(i), 1, '|');
				AfxExtractSubString (sRemoteFileSize,	sRemoteFileList.GetAt(i), 2, '|');
				
				// Build up the local path!
				sLocalFolder = m_sLocalFolder;
				if (!sRemoteFolder.IsEmpty ())
					sLocalFolder += (sRemoteFolder.Right (sRemoteFolder.GetLength () - nReferenceFolder));
				sLocalFolder.Replace ("/", "\\");
				
				m_dwFileLength = atoi (sRemoteFileSize);
				
				if (!sRemoteFileName.IsEmpty ())
				{
					sRemotePath = sRemoteFolder;
					StringHelper::AddBSToFolderName (sRemotePath, '/');
					sRemotePath += sRemoteFileName;
					
					// start receiving file
					if (StringHelper::WildcardCompare (sRemoteWildcard, sRemotePath))
						DownloadSingleFile (sRemotePath, sLocalFolder, pbAbort);
				}
				else
				{
					CreateMultiDirectory (sLocalFolder);
				}
			}
		}
	}
	catch (CFTPException * pEx)
	{
		// Dont catch CFTPException's! Let the caller catch it...
		throw;
	}
	catch (CFileException * pEx)
	{ 	
		// file can't be opened
		CString sTemp;
		sTemp.Format ("File Error - Filename = \"%s\", Error Code: %d", pEx->m_strFileName, pEx->m_cause);
		pEx->Delete ();
		throw new CFTPException (CFTPException::FTP_Local_UnknownFileError, sTemp);
	}
	catch (CException *pEx)
	{
		pEx->Delete ();
		throw new CFTPException (CFTPException::FTP_Unknown, "An unknown error occured");
	}
	catch (...)
	{
		throw new CFTPException (CFTPException::FTP_Unknown, "An unknown error occured");
	}
}


// We are sure that sOrgDestDir does not contain any White Spaces in the end and beginning
void CFTPDownload::DownloadSingleFile (const CString &sOrgSource, const CString &sOrgDestDir, bool *pbAbort)
{
	CString sSource (sOrgSource);

	m_ProgressDlg.SetWindowTitle ("Receiving " + sSource);
	m_ProgressDlg.SetPos (0);
	m_ProgressDlg.SetSecondStatus ("");

	CString sCurrentFTPDir;
	
	// split folder and file (if complete path is given)
	int nPos = sSource.ReverseFind ('/');
	ASSERT (nPos != -1);
	if (nPos != -1)
	{
		sCurrentFTPDir = sSource.Left (nPos + 1);
		sSource = sSource.Mid (nPos + 1);
	}

	CString sDestPath (sOrgDestDir);
	if (!sDestPath.IsEmpty ())
		StringHelper::AddBSToFolderName (sDestPath);
	
	sDestPath += sSource;

	CString sTemp;

	// set current directory
	if (!m_pFTPClient->m_pFtpConnection->SetCurrentDirectory (sCurrentFTPDir))
	{
		sTemp.Empty ();
		DWORD dwLength = 255, dwError;
		InternetGetLastResponseInfo (&dwError, sTemp.GetBuffer (dwLength), &dwLength);
		sTemp.ReleaseBuffer();
		throw new CFTPException (CFTPException::FTP_Remote_SetCurDirFailed, sTemp); 
	}

	// Show source file name
	m_ProgressDlg.SetStatus (sSource);
	// Show destination file name
	m_ProgressDlg.SetSecondStatus ("to: " + sDestPath);

	CFile file;
	// open destination file
	if (file.Open(sDestPath, CFile::modeCreate | CFile::modeWrite, NULL) == FALSE)
		throw new CFTPException (CFTPException::FTP_Local_FileCreateFailed, "Unable to create file" + sDestPath);
	
	m_ProgressDlg.SetStatus ("Opening " + sSource);

	CInternetFile* pInternetFile = m_pFTPClient->m_pFtpConnection->OpenFile (sSource, GENERIC_READ, m_dwTransferType);
	if (!pInternetFile)
		throw new CFTPException (CFTPException::FTP_Remote_FileOpenFailed, "Unable to open remote file " + sSource);
	
	// set max to 100%
	m_ProgressDlg.SetUpper(100);

	char buffer[BUF_SIZE];
	unsigned int nRead = BUF_SIZE;
	unsigned int nTotalRead = 0;
	while (	 nRead == BUF_SIZE && (WaitForSingleObject(m_hEventKill, 0) == WAIT_TIMEOUT) && 
			 (!(*pbAbort)))
	{
		// read remote data into buffer
		nRead = pInternetFile->Read (buffer, BUF_SIZE);
		// write buffer to data file
		file.Write (buffer, nRead);
		nTotalRead += nRead;
		
		sTemp.Format ("%s (%d of %d  KBytes transferred)", sSource, nTotalRead / 1024, m_dwFileLength / 1024);
		m_ProgressDlg.SetStatus (sTemp);

		if (m_dwFileLength > 0)
		{
			int nPos = (nTotalRead * 100) / m_dwFileLength;
			m_ProgressDlg.SetPos (nPos);
		}
		else
		{
			m_ProgressDlg.SetPos (0);
		}
	}
	
	// close the file
	file.Close ();
	pInternetFile->Close ();
	delete pInternetFile;

	if (WaitForSingleObject (m_hEventKill, 0) != WAIT_TIMEOUT)
		throw new CFTPException (CFTPException::FTP_UserAborted, "File transfer is aborted by the user");
}

bool CFTPDownload::CreateMultiDirectory (const CString &sDir)
{
	HANDLE		fFile;				// File Handle
	WIN32_FIND_DATA	fileinfo;		// File Information Structure
	CStringArray	m_arr;			// CString Array to hold Directory Structures
	BOOL tt;						// BOOL used to test if Create Directory was successful
	int x1 = 0;						// Counter
	CString tem = "";				// Temporary CString Object
	
	
	//Before we go to a lot of work. Check to see if the Directory exists
	fFile = FindFirstFile (sDir,&fileinfo);
	
	// if the file exists and it is a directory
	if(fileinfo.dwFileAttributes == FILE_ATTRIBUTE_DIRECTORY)
	{
		//  Directory Exists close file and return
		FindClose(fFile);
		return true;
	}
	m_arr.RemoveAll();					// Not really necessary - Just habit
	
	//Separate the String into its compounded components
	for(x1 = 0; x1 < sDir.GetLength(); x1++ )		// Parse the supplied CString Directory String
	{									
		if(sDir.GetAt(x1) != '\\')		// if the Charachter is not a \ 
			tem += sDir.GetAt(x1);		// add the character to the Temp String
		else
		{
			m_arr.Add(tem);				// if the Character is a \ 
			// Add the Temp String to the CString Array
			tem += "\\";				// Now add the \ to the temp string
		}
		if(x1 == sDir.GetLength()-1)		// If we reached the end of the String
			//add the remaining string
			m_arr.Add(tem);
	}
	
	// Close the file
	FindClose(fFile);
	
	//Create each level of the Directory Structure 
	// Now lets cycle through the String Array and create each directory in turn
	for(x1 = 1; x1 < m_arr.GetSize(); x1++)
	{
		tem = m_arr.GetAt(x1);
		tt = CreateDirectory(tem,NULL);
		
		// If the Directory exists it will return a false
		if(tt)
			SetFileAttributes(tem,FILE_ATTRIBUTE_NORMAL);
		// If we were successful we set the attributes to normal
	}
	//  Now lets see if the directory was successfully created
	fFile = FindFirstFile(sDir,&fileinfo);
	
	//Check to see if the Directory was created and it actually is a Directory
	m_arr.RemoveAll();
	if(fileinfo.dwFileAttributes == FILE_ATTRIBUTE_DIRECTORY)
	{
		//  Directory Exists close file and return
		FindClose(fFile);
		return true;
	}
	else
	{
		// For Some reason the Function Failed  Return FALSE
		FindClose(fFile);
		return false;
	}
}

void CFTPDownload::WaitForProgressDialog ()
{
	if(m_ProgressDlg.m_hWnd == NULL)
	{
		while(m_ProgressDlg.m_hWnd == NULL)
		{
			DoEvents();
		}
	}
	if(!::IsWindow(m_ProgressDlg.m_hWnd))
	{
		while(!::IsWindow(m_ProgressDlg.m_hWnd))
		{
			DoEvents();
		}
	}
}
