/*********************************************************************
	Copyright (C) 2002 Mustafa Demirhan :)
	
	This software is provided 'as-is', without any express or implied
	warranty.  In no event will the authors be held liable for any damages
	arising from the use of this software.

	Permission is granted to anyone to use this software for any purpose,
	including commercial applications, and to alter it and redistribute it
	freely, subject to the following restrictions:

	1. The origin of this software must not be misrepresented; you must not
	   claim that you wrote the original software. If you use this software
	   in a product, an acknowledgment in the product documentation would be
	   appreciated but is not required.

	2. Altered source versions must be plainly marked as such, and must not be
	   misrepresented as being the original software.

	3. This notice may not be removed or altered from any source distribution.

	http://www.macroangel.com
	admin@macroangel.com
**********************************************************************/

#include "stdafx.h"
#include "FTPUpload.h"
#include "FTPClient.h"
#include "StringHelper.h"
#include "FileInfo.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


CFTPUpload::CFTPUpload (CFTPClient *pClient)
{
	m_pFTPClient = pClient;
	
	ASSERT (m_pFTPClient);

	m_dwFileLength = 0;

	// default to binary transfer
	m_dwTransferType = FTP_TRANSFER_TYPE_BINARY;

	m_hEventKill = CreateEvent (NULL, TRUE, FALSE, NULL);
}

CFTPUpload::~CFTPUpload()
{
	m_ProgressDlg.DestroyWindow ();
	CloseHandle (m_hEventKill);
}


void CFTPUpload::UploadAllFiles (bool *pbAbort, bool bRecursive)
{
	// set animation to 'Uploading'
	m_ProgressDlg.m_nAnimationID = IDR_DOWNLOAD_ANIMATION;

	// Create the progress dialog box.
	if (!m_ProgressDlg.Create (NULL, m_hEventKill))
	{
		ASSERT (FALSE);
		throw new CFTPException (CFTPException::FTP_Unknown, "Unable to create the progress dialog. Please contact the program vendor for more information.");
	}
	
	WaitForProgressDialog ();

	// catch all exceptions
	try	
	{
		StringHelper::RemoveWSFromFileName (m_sLocalName);
		m_sLocalName.Replace		("/", "\\");
		
		StringHelper::RemoveWSFromFileName (m_sRemoteFolder);
		StringHelper::AddBSToFolderName	(m_sRemoteFolder, '/');
		m_sRemoteFolder.Replace		("\\", "/");
		
		StringHelper::RemoveWSFromFileName (m_sFTPRootFolder);
		StringHelper::AddBSToFolderName (m_sFTPRootFolder, '/');
		m_sFTPRootFolder.Replace	("\\", "/");

		
		CString sLocalWildcard, sBaseLocalFolder;
		m_pFTPClient->SeperateFilename (m_sLocalName, sBaseLocalFolder, sLocalWildcard, '\\');
		
		if (sBaseLocalFolder.IsEmpty ())
			throw new CFTPException (CFTPException::FTP_InvalidLocalFileName, "Local folder name is empty!");
		
		if (sLocalWildcard.IsEmpty ())
			sLocalWildcard = "*";

			
		// if a single file!
		if (sLocalWildcard.FindOneOf ("*?") == -1)
		{
			UploadSingleFile (m_sLocalName, m_sFTPRootFolder + m_sRemoteFolder, pbAbort);
		}
		// For multiple files!
		else
		{
			m_ProgressDlg.SetWindowTitle ("Receiving local directory listing...");
			m_ProgressDlg.SetSecondStatus ("");
			m_ProgressDlg.SetStatus ("Receiving local directory listing...");
			m_ProgressDlg.SetPos (0);
			
			// The first folder is our BASE folder!
			int nReferenceFolder = sBaseLocalFolder.GetLength ();

			CFileInfoArray fia;
			fia.AddDir (sBaseLocalFolder, sLocalWildcard, bRecursive, CFileInfoArray::AP_NOSORT, TRUE);
			
			CString sBaseRemoteFolder (m_sFTPRootFolder);
			m_pFTPClient->CreateMultiFolder (m_sRemoteFolder);
			sBaseRemoteFolder += m_sRemoteFolder;
			
			m_ProgressDlg.SetWindowTitle ("Creating remote directories...");
			m_ProgressDlg.SetStatus ("Creating remote directories...");

			// First create the directories
			for (int i = 0 ; i < fia.GetSize () ; i++)
			{
				if (fia [i].IsDirectory ())
				{
					CString sLocalFolder (fia [i].GetFilePath ());
					CString sRemoteFolder (m_sRemoteFolder);
					
					if (sLocalFolder.GetLength () > nReferenceFolder)
						sRemoteFolder += (sLocalFolder.Right (sLocalFolder.GetLength () - nReferenceFolder));

					m_pFTPClient->CreateMultiFolder (sRemoteFolder);
				}
			}

			for (i = 0 ; i < fia.GetSize () ; i++)
			{
				if (!fia [i].IsDirectory ())
				{
					CString sLocalFolder (fia [i].GetFileRoot ());
					CString sRemoteFolder (sBaseRemoteFolder);
					
					if (sLocalFolder.GetLength () > nReferenceFolder)
						sRemoteFolder += (sLocalFolder.Right (sLocalFolder.GetLength () - nReferenceFolder));
					
					UploadSingleFile (fia [i].GetFilePath (), sRemoteFolder, pbAbort);
				}
			}
		}
	}
	catch (CFTPException * pEx)
	{
		// Dont catch CFTPException's! Let the step catch it...
		throw;
	}
	catch (CFileException * pEx)
	{ 	
		// file can't be opened
		CString sTemp;
		sTemp.Format ("File Error - Filename = \"%s\", Error Code: %d", pEx->m_strFileName, pEx->m_cause);
		pEx->Delete ();
		throw new CFTPException (CFTPException::FTP_Local_UnknownFileError, sTemp);
	}
	catch (CException *pEx)
	{
		pEx->Delete ();
		throw new CFTPException (CFTPException::FTP_Unknown, "An unknown error occured");
	}
	catch (...)
	{
		throw new CFTPException (CFTPException::FTP_Unknown, "An unknown error occured");
	}
}

void CFTPUpload::UploadSingleFile (const CString &sOrgSource, const CString &sOrgDestDir, bool *pbAbort)
{
	m_ProgressDlg.SetWindowTitle ("Sending " + sOrgSource);
	m_ProgressDlg.SetPos (0);
	m_ProgressDlg.SetSecondStatus ("");

	CString sTemp;
	// set current directory
	if (!m_pFTPClient->m_pFtpConnection->SetCurrentDirectory (sOrgDestDir))
	{
		sTemp.Empty ();
		DWORD dwLength = 255, dwError;
		InternetGetLastResponseInfo (&dwError, sTemp.GetBuffer (dwLength), &dwLength);
		sTemp.ReleaseBuffer();
		throw new CFTPException (CFTPException::FTP_Remote_SetCurDirFailed, sTemp); 
	}

	// Show source file name
	m_ProgressDlg.SetStatus (sOrgSource);
	m_ProgressDlg.SetSecondStatus("to: " + sOrgDestDir);

	CFile file;
	// open source file
	if (file.Open (sOrgSource, CFile::modeRead, NULL) == FALSE)
		throw new CFTPException (CFTPException::FTP_Local_FileOpenFailed, "Unable to open file" + sOrgSource);

	m_ProgressDlg.SetStatus ("Opening " + sOrgDestDir + file.GetFileName ());

	CInternetFile* pInternetFile = m_pFTPClient->m_pFtpConnection->OpenFile (file.GetFileName (), GENERIC_WRITE);
	if (!pInternetFile)
		throw new CFTPException (CFTPException::FTP_Remote_FileWriteFailed, "Unable to open remote file " + sOrgDestDir + file.GetFileName () + " for writing");
	
	// set max position to 100%
	m_ProgressDlg.SetUpper (100);

	// get file size
	m_dwFileLength = file.GetLength ();

	char buffer [BUF_SIZE];
	unsigned int nRead = BUF_SIZE;
	unsigned int nTotalRead = 0;
	while (  nRead == BUF_SIZE && (WaitForSingleObject(m_hEventKill, 0) == WAIT_TIMEOUT) &&
			(!(*pbAbort)))
	{
		// read data into buffer
		nRead = file.Read (buffer, BUF_SIZE);
		// write buffer to remote data file
		pInternetFile->Write (buffer, nRead);
		nTotalRead += nRead;
		
		sTemp.Format ("%s (%d of %d  KBytes transferred)", sOrgSource, nTotalRead / 1024, m_dwFileLength / 1024);
		m_ProgressDlg.SetStatus (sTemp);

		// update progressbar
		if (m_dwFileLength > 0)
		{
			int nPos = (nTotalRead * 100) / m_dwFileLength;
			m_ProgressDlg.SetPos (nPos);
		}
		else
		{
			m_ProgressDlg.SetPos(0);
		}
	}

	// close the file and the internet file
	file.Close();
	pInternetFile->Close();
	delete pInternetFile;
	
	if (WaitForSingleObject (m_hEventKill, 0) != WAIT_TIMEOUT)
		throw new CFTPException (CFTPException::FTP_UserAborted, "File transfer is aborted by the user");
}

void CFTPUpload::WaitForProgressDialog ()
{
	if(m_ProgressDlg.m_hWnd == NULL)
	{
		while (m_ProgressDlg.m_hWnd == NULL)
		{
			DoEvents ();
		}
	}
	if(!::IsWindow(m_ProgressDlg.m_hWnd))
	{
		while (!::IsWindow(m_ProgressDlg.m_hWnd))
		{
			DoEvents ();
		}
	}
}
